
/**
 * Calculates either random or stores provided base colour and 
 * random shade of the same colour to apply on (plot) border
 * @param base
 * @returns
 */
function Colour (base) {
	this.background;
	this.border;
	/**
	 * Format RGBA colour
	 * @param number RED
	 * @param number GREEN
	 * @param number BLUE
	 * @param number alpha opacity; optional
	 */
	this.format = function(r,g,b,a) {
		var o = Math.round;
		if (a) { return 'rgba(' + o(r) + ',' + o(g) + ',' + o(b) + ',' + o(a) + ')'; }
		return 'rgb(' + o(r) + ',' + o(g) + ',' + o(b) + ')';
	}
	/**
	 * Randomise RGB colour
	 */
	this.randomRgba = function() {
	    var r = Math.random, s = 255;
	    return this.format(r()*s,r()*s,r()*s);
	}
	/**
	 * Calculates shade of provided colour, supported formats
	 * are hexadecimal #123456 or rgba(100,100,100) representations 
	 */
	this.shadeOf = function(baseColour) {
		var r, g, b, rgb;
		
		if (baseColour.length > 5 && baseColour.length < 10) {
			rgb = new RegExp("#?([A-Fa-f0-9]{2})([A-Fa-f0-9]{2})([A-Fa-f0-9]{2})")
				.exec(baseColour);
			
			if (rgb != null) {
				r = parseInt(rgb[1],16);
				g = parseInt(rgb[2],16);
				b = parseInt(rgb[3],16);
			} else {
				return this.randomRgba();
			}
		} else {
			rgb = new RegExp('rgba?\\(\\s*([0-9]+)\\s*,\\s*([0-9]+)\\s*,\\s*([0-9]+)\\s*')
				.exec(baseColour);
			
			if (rgb != null) {
				r = parseInt(rgb[1],10);
				g = parseInt(rgb[2],10);
				b = parseInt(rgb[3],10);
			} else {
				return this.randomRgba();
			}
		}
		
		//var max = Math.max(r,Math.max(g,b));
		//var step = (255 / max) * Math.random();
		var step = Math.random();
		
		r = (r * step) % 256;
		g = (g * step) % 256;
		b = (b * step) % 256;
		
		return this.format(r,g,b);
	}
	
	this.init = function(baseColour) {
		this.background = baseColour ? baseColour : this.randomRgba();
		this.border = this.shadeOf(this.background);
	}
	
	this.init(base);
}

/**
 * Class facilitating injection of chartsjs renders to provided container
 * @param target container, where charts should be plotted; argument is expected to be jQuery wrapped
 * @returns
 */
function Charts (target) {
	this.container = target;
	/**
	 * Stacked bar chart of job statuses per project
	 * @param array of project / status / count 
	 */
	this.plotExecutionChart = function(data) {
		var datasets = []; // Chart datasets
		var datapoints = {}; // Identified projects
		var labels = [];
		var types = {
			'FINISHED_OK': '#14ce53',
			'ERROR': '#ff5151',
			'N_A': '#edcc0b',
			'UNKNOWN': '#edcc0b',
			'RUNNING': '#338cf1'
		};
		/*
		 * Build dataset configuration
		 */
		for (var i=0;i<data.length;i++) {
			var entry = data[i];
			
			if (labels.indexOf(entry.project) < 0) {
				labels.push(entry.project);
			}
			
			if (typeof datapoints[entry.status] == 'undefined') {
				datapoints[entry.status] = {
					label: entry.status,
					colour: new Colour(types[entry.status] ? types[entry.status] : null),
					data: {}
				};
			}
			
			datapoints[entry.status]
				.data[entry.project] = entry.jobs;
		}
		/*
		 * Reformat configuration
		 */
		for (var stat in datapoints) {
			var config = datapoints[stat];
			var values = [];
			
			for (var i=0;i<labels.length;i++) {
				values.push(typeof config.data[labels[i]] == 'undefined' ? 
					0 : config.data[labels[i]]
				);
			}
			
			datasets.push({
				label : config.label,
				backgroundColor : config.colour.background,
				borderColor : config.colour.border,
				data : values
			});
		}
		
		this.plot({
			type: 'bar',
			data: {
				labels : labels,
				datasets : datasets
			},
			options: {
				title: {
					display: true,
					text: 'Job status by project'
				},
				scales: {
					xAxes: [{
						stacked: true
					}],
					yAxes: [{
						stacked: true
					}]
				}
			}
		},"50%");
	}
	/**
	 * Pie chart of components and their error count
	 * @param array of component types / failure counts
	 */
	this.plotComponentErrors = function (data) {
		var values = [];
		var labels = [];
		// Split data and labels
		for (var i=0;i<data.length;i++) {
			labels.push(data[i].component);
			values.push(data[i].failures);
		}
		// Plot new chart
		this.plot({
			type: 'pie',
			data: {
				labels: labels,
				datasets: [{
					data: values
				}],
			},
			options: {
				legend: false,
				cutoutPercentage: 50,
				title: {
					display: true,
					text: 'Errors by Component'
				},
				elements: {
					arc: {
						backgroundColor: (ctx) => (new Colour()).background
					}
				}
			}
		});
	}
	/**
	 * Plots chart, based on configuration to dashboard; optionally width of new plot can be specified
	 * @param object corresponding to 2nd Chart() argument https://www.chartjs.org/docs/latest/configuration/
	 * @param string width units (e.g. 20% or 300px)
	 */
	this.plot = function (chart,width) {
		var canvas = $('<div><canvas /></div>');
		
		if (width) {
			canvas.css("width", width);
		}
		
		this.container.append(canvas);
		new Chart($('canvas',canvas).get(0).getContext('2d'), chart);
	}
}

var charts = new Charts($('#dashboard'));

$.ajax({
	'method': 'GET',
	'url': '../stats',
	'dataType': 'json',
	'success': function(o) {
		charts.plotExecutionChart(o.jobsPerProject);
		charts.plotComponentErrors(o.failByComponent);
	}
});
